/*
 * This file is part of W3Olista, the statistics gathering program for
 * the WWW.                  This sourcecode is (c) Frank Pilhofer '95
 *
 * It is  distributed without warranty. The complete license terms for
 * this software  are included in  the file 'License.html'  in the doc
 * subdirectory. If  your copy came  without this file, you should get
 * it on the Web at    http://www.uni-frankfurt.de/~fp/Tools/W3Olista/
 * or contact the author,               fp@informatik.uni-frankfurt.de
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <dirent.h>
#include <time.h>
#include "fileio.h"
#include "cgiproc.h"

char momlogfile[512];
int  momlogline;

static char *months[] = {
  "Jan", "Feb", "Mar", "Apr", "May", "Jun",
  "Jul", "Aug", "Sep", "Oct", "Nov", "Dec",
  NULL
};

/*
 * programs to cat etc
 */
#ifdef CAT
static char *catprog   = CAT;
#else
static char *catprog   = NULL;
#endif
#ifdef ZCAT
static char *zcatprog  = ZCAT;
#else
static char *zcatprog  = NULL;
#endif
#ifdef GZCAT
static char *gzcatprog = GZCAT;
#else
static char *gzcatprog = NULL;
#endif

/*
 * Let's try making a faster one. Not as tolerant, though
 */

int scanstr (char *str, char *hostname, char *filename, char *when, int *status, int *size)
{
  static char temp[256];
  struct tm date, *dval;
  int count;
  time_t cdate;

  memset (&date, '\0', sizeof (struct tm));

  count = sscanf (str,
		  "%256[^ ] %*[^ ] %*[^ ] [%d/%[^/]/%d:%d:%d:%d%*[^\"]\"GET %512[^ ?]%*[^\"]\" %d %d",
		  hostname,
		  &date.tm_mday,
		  temp,           /* month name                     */
		  &date.tm_year,
		  &date.tm_hour,
		  &date.tm_min,
		  &date.tm_sec,
		  filename,       /* filename, we may have to parse */
		  status,
		  size);

  if (count != 10)
    return 0;

  for (count=0; months[count]; count++) {
    if (strncmp (temp, months[count], 3) == 0) {
      date.tm_mon = count;
      break;
    }
  }
  if (months[count]==NULL)
    return 0;
  date.tm_year -= 1900;

#ifndef GMT
  dval = &date;
#else
  date.tm_isdst = -1;
  cdate = mktime (&date);
  dval  = gmtime (&cdate);
#endif

  sprintf (when, "%d %s %d %02d %02d %02d",
	   dval->tm_year+1900, months[dval->tm_mon],
	   dval->tm_mday, dval->tm_hour, dval->tm_min, dval->tm_sec);

  /*
   * we only parse the filename if we have to. There are only few requests
   * that have to be parsed, so testing first should help speeding up.
   * most frequent have-to-parse case: requests upon "/%7Euser/" instead of
   * "/~user/".
   */

  if (strchr (filename, '%') != NULL)
    strcpy (filename, cgidecode (filename));

  return 1;
}

int numlogfiles (char *dirname, char *pcom, int (*acceptfile) (char *), int *lines)
{
  char cdname[256], filename[256], string[256], command[1024], *dptr, *catter;
  struct dirent *TheDirent;
  int res=0, type;
  DIR *TheDir;
  FILE *pipe;

  if (lines)
    *lines = 0;

  dptr = strtok (strcpy (cdname, dirname), "; ");

  while (dptr) {
#ifdef DEBUG
    fprintf (stderr, "numlogfiles() reading files from %s\n", dptr);
#endif
   if ((TheDir = opendir (dptr)) == NULL) {
      dptr = strtok (NULL, "; ");
      continue;
    }

    while ((TheDirent = readdir (TheDir)) != NULL) {
      strcpy (filename, TheDirent->d_name);
      
#ifdef DEBUG
    fprintf (stderr, "numlogfiles(): testing file %s\n", filename);
#endif
      if ((type = ((*acceptfile) (filename))) == LOGNOLOG)
	continue;
      
#ifdef DEBUG
      fprintf (stderr, "numlogfiles(): acceptfile() returned %d on %s\n", type, filename);
#endif

      switch (type) {
      case LOGPLAIN:
	catter = catprog;
	break;
      case LOGCOMPRESSED:
	catter = zcatprog;
	break;
      case LOGGZIPPED:
	catter = gzcatprog;
	break;
      default:
	catter = NULL;
	break;
      }

      res++;

      if (catter==NULL || *catter=='\0')
	continue;
/*      
 *     res++;
 */
      if (lines) {
	sprintf (command, "%s %s/%s %c %s %c wc -l 2> /dev/null",
		 catter, dptr, filename, '|',
		 (pcom && *pcom) ? pcom: "",
		 (pcom && *pcom) ? '|' : ' ');
#ifdef DEBUG
	fprintf (stderr, "numlogfiles(): using pipe %s\n", command);
#endif
	if ((pipe=popen (command, "r")) == NULL)
	  continue;
	if (fgets (string, 256, pipe) != NULL)
	  *lines += atoi (string);
	
	pclose (pipe);
      }
    }
    closedir (TheDir);
    
    dptr = strtok (NULL, "; ");
  }
  
  return res;
}    

int readlogfiles (char *dirname, char *pcom, int (*acceptfile) (char *), void (*action) (char *))
{
  char cdname[256], filename[256], string[1024], command[1024], *dptr, *catter;
  struct dirent *TheDirent;
  int res=0, type, pflag;
  DIR *TheDir;
  FILE *datei;

  dptr = strtok (strcpy (cdname, dirname), "; ");

  while (dptr) {
#ifdef DEBUG
    fprintf (stderr, "readlogfiles() reading files from %s\n", dptr);
#endif
    if ((TheDir = opendir (dptr)) == NULL) {
      dptr = strtok (NULL, "; ");
      continue;
    }
    
    while ((TheDirent = readdir (TheDir)) != NULL) {
      strcpy (filename, TheDirent->d_name);
      
      if ((type = ((*acceptfile) (filename))) == LOGNOLOG)
	continue;
#ifdef DEBUG
      fprintf (stderr, "readlogfiles(): acceptfile() returned %d on %s\n", type, filename);
#endif

      switch (type) {
      case LOGPLAIN:
	catter = catprog;
	break;
      case LOGCOMPRESSED:
	catter = zcatprog;
	break;
      case LOGGZIPPED:
	catter = gzcatprog;
	break;
      default:
	catter = NULL;
	break;
      }
      if (!(pcom && *pcom)) {
	if (type == LOGPLAIN) {
	  sprintf (command, "%s/%s", dptr, filename);
	  datei = fopen (command, "r");
	  pflag = 0;
	}
	else {
	  if (catter==NULL || *catter=='\0')
	    continue;
	  
	  sprintf (command, "%s %s/%s 2> /dev/null", catter, dptr, filename);
	  datei = popen (command, "r");
	  pflag = 1;
	}
      }
      else {
	if (catter==NULL || *catter=='\0')
	  continue;
	sprintf (command, "%s %s/%s %c %s 2> /dev/null",
		 catter, dptr, filename, 
		 (pcom && *pcom) ? '|' : ' ',
		 (pcom && *pcom) ? pcom: "");
	datei = popen (command, "r");
	pflag = 1;
      }
#ifdef DEBUG
      fprintf (stderr, "readlogfiles(): using %s on command %s\n", (pflag)?"pipe":"plain", command);
#endif
      if (datei == NULL)
	continue;
      
      sprintf (momlogfile, "%s/%s", dptr, filename);
      momlogline = 0; res++;
      
      while (!feof (datei)) {
	if (fgets (string, 1024, datei) == NULL)
	  break;
	
	momlogline++;
	(*action) (string);
      }
      if (pflag)
	pclose (datei);
      else
	fclose (datei);
    }
    closedir (TheDir);

    dptr = strtok (NULL, "; ");
  }
  
  return res;
}  
