/*
 * This file is part of W3Olista, the statistics gathering program for
 * the WWW.                  This sourcecode is (c) Frank Pilhofer '95
 *
 * It is  distributed without warranty. The complete license terms for
 * this software  are included in  the file 'License.html'  in the doc
 * subdirectory. If  your copy came  without this file, you should get
 * it on the Web at    http://www.uni-frankfurt.de/~fp/Tools/W3Olista/
 * or contact the author,               fp@informatik.uni-frankfurt.de
 */


/*
 * The HTML part of Olista
 */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <memory.h>
#include "cgiproc.h"
#include "getcgi.h"
#include "struct.h"
#include "html.h"
#include "project.h"
#include "domains.h"

/*
 * Internal functions
 */

void		pCGIprev	(FILE *, int);
void		pCGIself	(FILE *, void *);
void		pCGIdisp   	(FILE *, int);
void		pCGIsort	(FILE *, int);
void		pCGIdepth	(FILE *, int);
void		pCGIrep		(FILE *, int);
void		pCGIall		(FILE *, void *, int, int, int, int, int);
void		HTMLphosts	(FILE *, int, host *,  int);
void		HTMLpfiles	(FILE *, int, file *,  int);
void		HTMLpdates	(FILE *, int, dtree *, int);
void		HTMLpents	(FILE *, int, entry *, int);
void		HTMLphsum	(FILE *, int, host *);
void		HTMLpfsum	(FILE *, int, file *,  int);
void		HTMLpdsum	(FILE *, int, dtree *, int);

/*
 * Default values for Forms
 */

#ifndef INHOST
#define INHOST ""
#endif
#ifndef EXHOST
#define EXHOST ""
#endif
#ifndef INFILE
#define INFILE ""
#endif
#ifndef EXFILE
#define EXFILE ""
#endif

static char   *definhost = INHOST;
static char   *defexhost = EXHOST;
static char   *definfile = INFILE;
static char   *defexfile = EXFILE;
static char   *format    = "PRE";

/*
 * print HTML
 */

void htmlheader (FILE *datei)
{
  FILE *custom;
  char *p1, temp[256];
  int count=1;

  if (GetInputValue ("SCRIPT_NAME"))
    fprintf (datei, "Content-type: text/html\n\n");

  if ((p1 = GetInputValue ("CustomHeader")) != NULL) {
    if ((custom = fopen (cgidecode(p1), "r")) != NULL) {
      while (!feof (custom)) {
	if ((count = fread (temp, 1, 256, custom)) <= 0)
	  break;
	fwrite (temp, 1, count, datei);
      }
      fclose (custom);
    }
  }
  if (p1 == NULL || custom == NULL) {
    fprintf (datei, "\
<html><head>\n\
<title>W3OLISTA - Online WWW statistics</title>\n\
</head>\n\
<body>\n\
<h1>W3OLISTA - Online WWW statistics</h1>\n\
");
  }
}

void PrintForm (FILE *datei)
{
  char *ptr, **iter;
  char d1[256], d2[256];

  fprintf (datei, "\n<FORM METHOD=\"POST\" ACTION=\"");

  if ((ptr = GetInputValue ("SCRIPT_NAME")) != NULL)
    fprintf (datei, "%s/cgi", ptr);

  if (strcmp (logdir, dlogdir) != 0)
    fprintf (datei, "/logdir=%s",  cgiencode (logdir));
  if (strcmp (logfile, dlogfile) != 0)
    fprintf (datei, "/logfile=%s", cgiencode (logfile));
  /*
   * Don't allow address lookup when CGI. Takes way too long.
   *
   * if (resolveaddr)
   *   fprintf (datei, "/ResolveAddr=Yes");
   */
  if (cutprefix==0)
    fprintf (datei, "/CutPrefix=No");
  if (countunique)
    fprintf (datei, "/CountUnique=Yes");
  if ((ptr = GetInputValue ("CustomHeader")) != NULL)
    fprintf (datei, "/CustomHeader=%s", ptr);
  if ((ptr = GetInputValue ("CustomFooter")) != NULL)
    fprintf (datei, "/CustomFooter=%s", ptr);

  fprintf (datei, "\">\n\
<hr>\
<KBD>Include hosts: </KBD><input name=\"inhost001\" size=\"40\" value=\"");

  if ((iter=inhost) && *inhost && **inhost) {
    fprintf (datei, "%s", *iter++);
    while (*iter) fprintf (datei, ", %s", *iter++);
  }
  else
    fprintf (datei, "%s", definhost);

  fprintf (datei, "\"><br>\n\
<KBD>Exclude hosts: </KBD><input name=\"exhost001\" size=\"40\" value=\"");

  if ((iter=exhost) && *exhost && **exhost) {
    fprintf (datei, "%s", *iter++);
    while (*iter) fprintf (datei, ", %s", *iter++);
  }
  else
    fprintf (datei, "%s", defexhost);

  fprintf (datei, "\"><br>\n\
<KBD>Include files: </KBD><input name=\"infile001\" size=\"40\" value=\"");

  if ((iter=infile) && *infile && **infile) {
    fprintf (datei, "%s", cgidecode (*iter++));
    while (*iter) fprintf (datei, ", %s", cgidecode (*iter++));
  }
  else
    fprintf (datei, "%s", definfile);

  fprintf (datei, "\"><br>\n\
<KBD>Exclude files: </KBD><input name=\"exfile001\" size=\"40\" value=\"");

  if ((iter=exfile) && *exfile && **exfile) {
    fprintf (datei, "%s", cgidecode (*iter++));
    while (*iter) fprintf (datei, ", %s", cgidecode (*iter++));
  }
  else
    fprintf (datei, "%s", defexfile);

  if (repwhat == RDATES) {
    sprintf (d1, "%s %02d %02d",
	     mnames[fromdate.tm_mon+1], fromdate.tm_mday, fromdate.tm_year);
    sprintf (d2, "%s %02d %02d",
	     mnames[todate.tm_mon+1],   todate.tm_mday, todate.tm_year);
  }

  fprintf (datei, "\"><br>\n<hr>\n");
  fprintf (datei, "\
<p>\n\
Create a report on \n\
<select name=\"Report\">\n\
<option %s value=\"%s\">Today\n\
<option %s value=\"%s\">Yesterday\n\
<option %s value=\"%s\">This Week\n\
<option %s value=\"%s\">Last Week\n\
<option %s value=\"%s\">This Month\n\
<option %s value=\"%s\">Last Month\n\
<option %s value=\"%s\">Last 24 Hours\n\
<option %s value=\"%s\">Last 7 Days\n\
<option %s value=\"%s\">Everything\n\
<option %s value=\"%s\">These Dates:\n\
</select>\n\
",
	   (repwhat == RTODAY)      ? "selected" : "", repnames[RTODAY],
	   (repwhat == RYESTERDAY)  ? "selected" : "", repnames[RYESTERDAY],
	   (repwhat == RTHISWEEK)   ? "selected" : "", repnames[RTHISWEEK],
	   (repwhat == RLASTWEEK)   ? "selected" : "", repnames[RLASTWEEK],
	   (repwhat == RTHISMONTH)  ? "selected" : "", repnames[RTHISMONTH],
	   (repwhat == RLASTMONTH)  ? "selected" : "", repnames[RLASTMONTH],
	   (repwhat == RLAST24HOURS)? "selected" : "", repnames[RLAST24HOURS],
	   (repwhat == RLAST7DAYS)  ? "selected" : "", repnames[RLAST7DAYS],
	   (repwhat == REVERYTHING) ? "selected" : "", repnames[REVERYTHING],
	   (repwhat == RDATES)      ? "selected" : "", repnames[RDATES]);

  fprintf (datei, "\
From <input name=\"FromDate\" size=\"10\" value=\"%s\">\n\
To <input name=\"ToDate\" size=\"10\" value=\"%s\">\n\
<br>\n\
",
	   (repwhat == RDATES)        ? d1 : "",
	   (repwhat == RDATES)        ? d2 : "");

  fprintf (datei, "\
Host Listing\n\
<select name=\"HostList\">\n\
<option %s value=\"yes\">Yes\n\
<option %s value=\"no\" >No\n\
<option %s value=\"sum\">Summary\n\
</select>\n\
Detail level:\n\
<select name=\"HostDetail\">\n\
<option %s value=\"0\">All Detail\n\
<option %s value=\"1\">One Level\n\
<option %s value=\"2\">Two Levels\n\
<option %s value=\"3\">Three Levels\n\
<option %s value=\"4\">Four Levels\n\
</select><br>\n",
	   ((display&HOSTLIST)==HOSTLIST)              ? "selected" : "",
	   ((display&HOSTLIST)==0)                     ? "selected" : "",
	   ((display&HOSTSUM) ==HOSTSUM)               ? "selected" : "",
	   (((maxdepth&HOSTDETMASK)>>HOSTDETOFF) == 0) ? "selected" : "",
	   (((maxdepth&HOSTDETMASK)>>HOSTDETOFF) == 1) ? "selected" : "",
	   (((maxdepth&HOSTDETMASK)>>HOSTDETOFF) == 2) ? "selected" : "",
	   (((maxdepth&HOSTDETMASK)>>HOSTDETOFF) == 3) ? "selected" : "",
	   (((maxdepth&HOSTDETMASK)>>HOSTDETOFF) == 4) ? "selected" : "");

  fprintf (datei, "\
File Listing\n\
<select name=\"FileList\">\n\
<option %s value=\"yes\">Yes\n\
<option %s value=\"no\" >No\n\
<option %s value=\"sum\">Summary\n\
</select>\n\
Detail level:\n\
<select name=\"FileDetail\">\n\
<option %s value=\"0\">All Detail\n\
<option %s value=\"1\">One Level\n\
<option %s value=\"2\">Two Level\n\
<option %s value=\"3\">Three Levels\n\
<option %s value=\"4\">Four Levels\n\
</select><br>\n",
	   ((display&FILELIST)==FILELIST)              ? "selected" : "",
	   ((display&FILELIST)==0)                     ? "selected" : "",
	   ((display&FILESUM) ==FILESUM)               ? "selected" : "",
	   (((maxdepth&FILEDETMASK)>>FILEDETOFF) == 0) ? "selected" : "",
	   (((maxdepth&FILEDETMASK)>>FILEDETOFF) == 1) ? "selected" : "",
	   (((maxdepth&FILEDETMASK)>>FILEDETOFF) == 2) ? "selected" : "",
	   (((maxdepth&FILEDETMASK)>>FILEDETOFF) == 3) ? "selected" : "",
	   (((maxdepth&FILEDETMASK)>>FILEDETOFF) == 4) ? "selected" : "");

  fprintf (datei, "\
Date Listing\n\
<select name=\"DateList\">\n\
<option %s value=\"yes\">Yes\n\
<option %s value=\"no\" >No\n\
<option %s value=\"sum\">Summary\n\
</select>\n\
Detail level:\n\
<select name=\"DateDetail\">\n\
<option %s value=\"0\">All Detail\n\
<option %s value=\"1\">Year\n\
<option %s value=\"2\">Month\n\
<option %s value=\"3\">Day\n\
<option %s value=\"4\">Hour\n\
</select><br>\n",
	   ((display&DATELIST)==DATELIST)              ? "selected" : "",
	   ((display&DATELIST)==0)                     ? "selected" : "",
	   ((display&DATESUM) ==DATESUM)               ? "selected" : "",
	   (((maxdepth&DATEDETMASK)>>DATEDETOFF) == 0) ? "selected" : "",
	   (((maxdepth&DATEDETMASK)>>DATEDETOFF) == 1) ? "selected" : "",
	   (((maxdepth&DATEDETMASK)>>DATEDETOFF) == 2) ? "selected" : "",
	   (((maxdepth&DATEDETMASK)>>DATEDETOFF) == 3) ? "selected" : "",
	   (((maxdepth&DATEDETMASK)>>DATEDETOFF) == 4) ? "selected" : "");

  fprintf (datei, "\
Entry Listing (exclusive)\n\
<select name=\"EntryList\">\n\
<option %s value=\"yes\">Yes\n\
<option %s value=\"no\" >No\n\
</select><br>\n",
	   ((display&ENTRYLIST)==ENTRYLIST)? "selected" : "",
	   ((display&ENTRYLIST)==0)        ? "selected" : "");

  fprintf (datei, "\
Sort\n\
<select name=\"Sort\">\n\
<option %s value=\"by-alpha\">alphabetically\n\
<option %s value=\"by-req\"  >by requests\n\
<option %s value=\"by-size\" >by size\n\
</select>\n\
<br>\n\
",
	   (sort    == SORTALPHA) ? "selected" : "",
	   (sort    == SORTREQ)   ? "selected" : "",
	   (sort    == SORTSIZE)  ? "selected" : "");

  fprintf (datei, "\
<p>\n\
<INPUT type=\"submit\" VALUE=\"Run Report\">\n\
</FORM>\n\
\n");
}

void HTMLSigHandler (int sig)
{
  char *p1;

  fprintf (stdout, "Content-type: text/html\n\n\
<h1>W3Olista crashed</h1>\n\
Congratulations! You have just <b>crashed</b> W3Olista. If a core dump\n\
was created, and you know how to operate a debugger, please check the\n\
following:\n\
<ul>\n\
  <li> <i>Where</i> the crash occured,\n\
  <li> The <i>environment</i> (local variables),\n\
  <li> <i>What</i> you were doing\n\
  <li> If the crash is <i>reproduceable</i>\n\
</ul>\n\
You can check if the crash is reproduceable by running the same report\n\
<a href=\"");
  pCGIall (stdout, NULL, 0, display, maxdepth, sort, repwhat);
  fprintf (stdout, "/CoreRepro\">again</a>.<br>\n\
Please use the following form to report the crash and its\n\
circumstances to the Author. Fill out the <i>name</i> and <i>email</i>\n\
fields if you want a reply. If you prefer staying anonymous, then don't.<br>\n\
<form method=\"POST\" ACTION=\"http://www.uni-frankfurt.de:83/~fp/cgi/sendmail/Olista\">\n\
<input name=\"CoreDump\" value=\"W3Olista\" type=hidden>\n\
Your name:  <INPUT NAME=\"name\" SIZE=\"48\"> <p>\n\
Your email: <INPUT NAME=\"email\" SIZE=\"48\"> <p>\n\
<p>\n\
Please describe everything describeable:<br>\n\
<TEXTAREA NAME=\"text\" ROWS=10 COLS=50>Hi Frank,\n\n\
Olista catched a signal (%d) as I used the following URL:\n", sig);

  if ((p1 = GetInputValue ("SERVER_NAME")) != NULL) {
    fprintf (stdout, "http://%s", p1);
    if ((p1 = GetInputValue ("SERVER_PORT")) != NULL) {
      if (atoi (p1) != 80)
	fprintf (stdout, ":%d", atoi (p1));
    }
  }
  pCGIall (stdout, NULL, 0, display, maxdepth, sort, repwhat);
  fprintf (stdout, "\n");
  if (GetInputValue ("CoreRepro")) 
    fprintf (stdout, "\nThe error _is_ reproduceable.\n");
  else
    fprintf (stdout, "\nI have not yet checked (couldn't check)\nif the error is reproduceable.\n");

  fprintf (stdout, "\n\
</TEXTAREA>\n\
<p>\n\
<INPUT type=\"submit\" VALUE=\"Submit Report\">\n\
</FORM>\n\
\n\
");
  htmlfooter (stdout);
  exit (0);
}

void PrintSendCom (FILE *datei)
{
  fprintf (datei, "\
<h2>Send a Comment</h2>\n\
Please insert your comments into the following form. Don't forget to\n\
include your email address, especially if you'd like a reply. If you\n\
have W3Olista up and running, please also mention your statistics\n\
base document (from where the statistics are accessible). I will not\n\
publish your URL.<p>\n\n\
");

  fprintf (datei, "\
<FORM METHOD=\"POST\" ACTION=\"http://www.uni-frankfurt.de:83/~fp/cgi/omail/To=fp@informatik.uni-frankfurt.de/Version=%s/Patch=%s\">\n\
<dl>\n\
  <dt> Your name: <dd> <INPUT NAME=\"name\"  SIZE=\"48\">\n\
  <dt> Your email:<dd> <INPUT NAME=\"email\" SIZE=\"48\">\n\
  <dt> Your comments: <dd>\n\
<TEXTAREA NAME=\"text\" ROWS=10 COLS=50>\n\
</TEXTAREA>\n\
  <dt> Your Statistics Page: <dd> <input name=\"web\" size=\"48\">\n\
<p>\n\
  <dt> <dd> <INPUT type=submit VALUE=\"Send Comment\">\n\
</dl>\n\
</FORM>\n\
\n\
",
	   version, patch);
}

void htmlfooter (FILE *datei)
{
  time_t now = time(NULL);
  char *ptr, temp[256];
  FILE *custom;
  int count;

  fprintf (datei, "\n\
<p>\n\
<i>W3OLISTA %s%s</i> %s.\n", version, patch, ctime (&now));

  if ((ptr = GetInputValue ("CustomFooter")) != NULL) {
    if ((custom = fopen (cgidecode(ptr), "r")) != NULL) {
      while (!feof (custom)) {
	if ((count = fread (temp, 1, 256, custom)) <= 0)
	  break;
	fwrite (temp, 1, count, datei);
      }
      fclose (custom);
    }
  }
  if (ptr==NULL || custom==NULL) {
    if (linktype==LREPLINK && (ptr = GetInputValue ("SCRIPT_NAME")))
      fprintf (datei, "\
Send a <a href=\"%s/cgi/sendcomment\">Comment</a> to the author.<br>\n",
	       ptr);

    fprintf (datei,"\
<hr>\n\
<address><a href=\"http://www.uni-frankfurt.de/~fp/Frank.html\">Frank Pilhofer</a> &lt;fp@informatik.uni-frankfurt.de&gt; <a href=\"http://www.uni-frankfurt.de/~fp/\">Frank's Homepage</a></address>\n\
</body> </html>\n\
");
  }
}

/*
 * Print a complete reference to myself
 */

void pCGIprev (FILE *datei, int excount)
{
  static char string[256];
  int index=1, rtype, count=0;

  while (1) {
    if ((rtype = GetCGIReport (index, string)) == NOTYPE)
      break;

    if (index!=excount) {
      switch (rtype) {
      case HOSTTYPE:
	fprintf (datei, "/rep%03d=host;%s", ++count, cgiencode (string));
	break;
      case FILETYPE:
	fprintf (datei, "/rep%03d=file;%s", ++count, cgiencode (string));
	break;
      case DATETYPE:
	fprintf (datei, "/rep%03d=date;%s", ++count, cgiencode (string));
	break;
      }
    }
    index++;
  }
}

void pCGIself (FILE *datei, void *exptr)
{
  char *ptr, **pptr;
  int index;

  if ((ptr = GetInputValue ("SCRIPT_NAME")) != NULL)
    fprintf (datei, "%s/cgi", ptr);
  else {
    fprintf (datei, "<h1>Error</h1>");
    return;
  }
  if (strcmp (logdir, dlogdir) != 0)
    fprintf (datei, "/logdir=%s",  cgiencode (logdir));
  if (strcmp (logfile, dlogfile) != 0)
    fprintf (datei, "/logfile=%s", cgiencode (logfile));
  if (resolveaddr)
    fprintf (datei, "/ResolveAddr=Yes");
  if (cutprefix==0)
    fprintf (datei, "/CutPrefix=No");
  if (countunique)
    fprintf (datei, "/CutUnique=Yes");
  if ((ptr = GetInputValue ("CustomHeader")) != NULL)
    fprintf (datei, "/CustomHeader=%s", ptr);
  if ((ptr = GetInputValue ("CustomFooter")) != NULL)
    fprintf (datei, "/CustomFooter=%s", ptr);

  if ((pptr=inhost) != NULL) {
    index=1;
    while (*pptr) {
      if (*pptr != exptr)
	fprintf (datei, "/inhost%03d=%s", index++, cgiencode (*pptr));
      pptr++;
    }
  }
  if ((pptr=exhost) != NULL) {
    index=1;
    while (*pptr) {
      if (*pptr != exptr)
	fprintf (datei, "/exhost%03d=%s", index++, cgiencode (*pptr));
      pptr++;
    }
  }
  if ((pptr=infile) != NULL) {
    index=1;
    while (*pptr) {
      if (*pptr != exptr)
	fprintf (datei, "/infile%03d=%s", index++, cgiencode (*pptr));
      pptr++;
    }
  }
  if ((pptr=exfile) != NULL) {
    index=1;
    while (*pptr) {
      if (*pptr != exptr)
	fprintf (datei, "/exfile%03d=%s", index++, cgiencode (*pptr));
      pptr++;
    }
  }
}

void pCGIdisp (FILE *datei, int dtype)
{
  if ((dtype&HOSTLIST)==HOSTLIST)    fprintf (datei, "/HostList=Yes");
  else if ((dtype&HOSTSUM)==HOSTSUM) fprintf (datei, "/HostList=Summary");

  if ((dtype&FILELIST)==FILELIST)    fprintf (datei, "/FileList=Yes");
  else if ((dtype&FILESUM)==FILESUM) fprintf (datei, "/FileList=Summary");

  if ((dtype&DATELIST)==DATELIST)    fprintf (datei, "/DateList=Yes");
  else if ((dtype&DATESUM)==DATESUM) fprintf (datei, "/DateList=Summary");

  if ((dtype&ENTRYLIST)==ENTRYLIST)  fprintf (datei, "/EntryList=Yes");
}

void pCGIsort (FILE *datei, int stype)
{
  if (stype==-1)
    return;

  switch (stype) {
  case SORTALPHA: /* This is the default */
                  fprintf (datei, "/Sort=by-alpha"); break;
  case SORTREQ:   fprintf (datei, "/Sort=by-req");   break;
  case SORTSIZE:  fprintf (datei, "/Sort=by-size");  break;
  }
}

void pCGIdepth (FILE *datei, int depth)
{
  if (depth==-1)
    return;

  if (depth&HOSTDETMASK)
    fprintf (datei, "/HostDetail=%d", (depth&HOSTDETMASK)>>HOSTDETOFF);
  if (depth&FILEDETMASK)
    fprintf (datei, "/FileDetail=%d", (depth&FILEDETMASK)>>FILEDETOFF);
  if (depth&DATEDETMASK)
    fprintf (datei, "/DateDetail=%d", (depth&DATEDETMASK)>>DATEDETOFF);
}

void pCGIrep (FILE *datei, int rtype)
{
  if (rtype==-1)
    return;

  fprintf (datei, "/Report=%s", repnames[rtype]);

  if (rtype==RDATES) {
    fprintf (datei, "/FromDate=%s%%20%02d%%20%02d",
	     mnames[fromdate.tm_mon+1], fromdate.tm_mday, fromdate.tm_year);
    fprintf (datei, "/ToDate=%s%%20%02d%%20%02d",
	     mnames[todate.tm_mon+1], todate.tm_mday, todate.tm_year);
  }
}

void pCGIall (FILE *datei, void *exself, int exprev, int disp, int depth, int ssort, int reptype)
{
  pCGIself (datei, exself);
  pCGIprev (datei, exprev);
  pCGIdisp (datei, disp);
  pCGIdepth(datei, depth);
  pCGIsort (datei, ssort);
  pCGIrep  (datei, reptype);
}

char *Space (int num, int c)
{
  static char stsp[1024];

  if (num < 0) num = 0;
  if (c <= 0)  c = ' ';

  memset (stsp, c, num);
  stsp[num] = '\0';

  return stsp;
}

/*
 * Print file/host/date trees in HTML format
 */

void HTMLphosts (FILE *datei, int numrep, host *list, int depth)
{
  static char fqstring[1024];
  static int crtype;
  int iter;

  if (depth == 0) {
    fprintf (datei, "<a name=\"HostListing\"><h3>Listing by Host</h3></a>\n");
    fprintf (datei, "<%s>", format);
    fprintf (datei, " Requests  kBytes : Host\n");
    fprintf (datei, "------------------:-%s\n",Space(barlength+baroffset,'-'));

    crtype = (display & ~HOSTLIST)|((display&FILESUM)?DATELIST:FILELIST);

    if (list==NULL) {
      fprintf (datei, "</%s>\n<h3>No data was found for this query</h3>\n<p>\n",
	       format);
      return;
    }
  }

  while (list) {
    fprintf (datei, " %8d %7ld : ", list->req, list->size>>1);
    for (iter=0; iter<depth; iter++)
      fprintf (datei, "  ");

    if (linktype==LREPLINK) {
      fprintf   (datei, "<a href=\"");
      pCGIall   (datei, NULL, 0, crtype, maxdepth, sort, repwhat);
      getfqhost (list, fqstring);
      fprintf   (datei, "/rep%03d=host;%s\"",
		 numrep, cgiencode (fqstring));

      if (depth == 0)
	fprintf (datei, " name=\"%s\"", fqstring);

      fprintf   (datei, ">%s</a>", list->name);

      if (depth == 0)
	fprintf (datei, "%s(%s)",
		 Space(baroffset-strlen(list->name),' '),
		 GetDomainTitle (list->name));

      fprintf   (datei, "\n");
    }
    else if (depth == 0) {
      getfqhost (list, fqstring);
      fprintf   (datei, "<a name=\"%s\">%s</a>%s(%s)\n",
		 fqstring, list->name,
		 Space(baroffset-strlen(list->name),' '),
		 GetDomainTitle (list->name));
    }
    else
      fprintf   (datei, "%s\n", list->name);

    if (list->child && (HOSTDEPTH(maxdepth)==0 || 
			HOSTDEPTH(maxdepth)!=depth+1) &&
	(depth!=0 || strcmp (list->name, "(num)")!=0))
      HTMLphosts (datei, numrep, list->child, depth+1);

    list = list->NEXT;
  }
  if (depth == 0)
    fprintf (datei, "</%s>\n", format);
}

void HTMLpfiles (FILE *datei, int numrep, file *list, int depth)
{
  static char fqstring[1024];
  static int crtype;
  int iter;

  if (depth == 0) {
    fprintf (datei, "<a name=\"FileListing\"><h3>Listing by File</h3></a>\n");
    fprintf (datei, "<%s>", format);
    fprintf (datei, " Requests  kBytes : File\n");
    fprintf (datei, "------------------:-%s\n",Space(barlength+baroffset,'-'));

    crtype = (display & ~FILELIST)|((display&HOSTSUM)?DATELIST:HOSTLIST);
    
    if (list==NULL) {
      fprintf (datei, "</%s>\n<h3>No data was found for this query</h3>\n<p>\n", format);
      return;
    }
  }

  while (list) {
    fprintf (datei, " %8d %7ld : ", list->req, list->size>>1);
    for (iter=0; iter<depth; iter++)
      fprintf (datei, "  ");

    if (linktype==LREPLINK) {
      fprintf   (datei, "<a href=\"");
      pCGIall   (datei, NULL, 0, crtype, maxdepth, sort, repwhat);
      getfqfile (list, fqstring);
      fprintf   (datei, "/rep%03d=file;%s\"",
		 numrep, cgiencode (fqstring));

      if (depth == 1)
	fprintf (datei, " name=\"%s\"", fqstring);

      fprintf   (datei, ">%s%s</a>\n", list->name, (list->isleaf)?"":"/");
    }
    else if (linktype==LFILELINK) {
      getfqfile (list, fqstring);
      fprintf   (datei, "<a href=\"%s://%s:%d%s%s\"",
		 servtype, servhost, servport, fqstring,
		 (list->child)?"/":"");

      if (depth == 1)
	fprintf (datei, " name=\"%s\"", fqstring);
      
      fprintf   (datei, ">%s%s</a>\n", list->name, (list->isleaf)?"":"/");
    }
    else if (depth == 1) {
      getfqfile (list, fqstring);
      fprintf   (datei, "<a name=\"%s\">%s%s</a>\n",
		 fqstring, list->name, (list->isleaf)?"":"/");
    }
    else
      fprintf   (datei, "%s%s\n", list->name, (list->isleaf)?"":"/");

    if (list->child && (FILEDEPTH(maxdepth)==0 ||
			FILEDEPTH(maxdepth)!=depth))
      HTMLpfiles (datei, numrep, list->child, depth+1);

    list = list->NEXT;
  }
  if (depth==0)
    fprintf (datei, "</%s>\n", format);
}

void HTMLpdates (FILE *datei, int numrep, dtree *list, int depth)
{
  static char fqstring[1024];
  static int maxreq=0, crtype;
  int iter;
  dtree *max;

  if (depth == 0) {
    if (list) {
      if ((max=(dtree *) getmaxreq (list,DATETYPE,((DATEDEPTH(maxdepth))?(DATEDEPTH(maxdepth)-1):3)))!=NULL)
	maxreq = max->req;
      else
	maxreq = 0;
    }

    fprintf (datei, "<a name=\"DateListing\"><h3>Listing by Date</h3></a>\n");
    fprintf (datei, "<%s>", format);
    fprintf (datei, " Requests  kBytes : Date%s",
	     Space (baroffset-strlen("Date"), ' '));
    if (maxreq&&barlength) {
      if (maxreq/barlength > 5)
	fprintf (datei,  "Requests (# = %d)", maxreq/barlength);
      else
	fprintf (datei,  "Requests (# = %d.%02d)", maxreq/barlength,
		 (100*maxreq)/barlength-100*(maxreq/barlength));
    }
    fprintf (datei, "\n");
    fprintf (datei, "------------------:-%s\n",Space(barlength+baroffset,'-'));

    crtype = (display & ~DATELIST)|((display&FILESUM)?HOSTLIST:FILELIST);

    if (list==NULL) {
      fprintf (datei, "</%s>\n<h3>No data was found for this query</h3>\n<p>\n", format);
      return;
    }
  }
  
  while (list) {
    fprintf (datei, " %8d %7ld : ", list->req, list->size>>1);
    for (iter=0; iter<depth; iter++)
      fprintf (datei, "  ");

    if (linktype==LREPLINK) {
      fprintf   (datei, "<a href=\"");
      pCGIall   (datei, NULL, 0, crtype, maxdepth, sort, repwhat);
      getfqdate (list, fqstring);
      fprintf   (datei, "/rep%03d=date;%s\"",
		 numrep, cgiencode (fqstring));

      if (depth == 2)
	fprintf (datei, " name=\"%s\"", fqstring);

      fprintf   (datei, ">%s</a>", list->name);
    }
    else if (depth == 2) {
      getfqdate (list, fqstring);
      fprintf   (datei, "<a name=\"%s\">%s</a>", fqstring, list->name);
    }
    else
      fprintf   (datei, "%s", list->name);

    if (depth == ((DATEDEPTH(maxdepth))?(DATEDEPTH(maxdepth)-1):3)) {
      fprintf (datei, "%s",  Space(baroffset-strlen(list->name)-2*depth,' '));
      fprintf (datei, "%s\n",Space(maxreq?(barlength*list->req/maxreq):0,'#'));
    }
    else
      fprintf (datei, "\n");

    if (list->child && depth!=3 &&
	(DATEDEPTH(maxdepth)==0 || 
	 DATEDEPTH(maxdepth)!=depth+1))
      HTMLpdates (datei, numrep, list->child, depth+1);
    
    list = list->NEXT;
  }
  if (depth==0)
    fprintf (datei, "</%s>\n", format);
}

void HTMLpents (FILE *datei, int numrep, entry *list, int depth)
{
  static char fqstring[1024];
  struct tm then;

  fprintf (datei, "<a name=\"EntryListing\"><h3>Entry Listing</h3></a>\n");

  if (list==NULL) {
    fprintf (datei, "<p>\n<h3>No data was found for this query</h3>\n<p>\n");
    return;
  }
  fprintf (datei, "<%s>", format);

  while (list) {
    dtreetotm (&then, getfqdate (list->edate, fqstring));
    fprintf   (datei, "%s %02d %02d %02d:%02d:%02d ",
	       mnames[then.tm_mon+1], then.tm_mday,
	       then.tm_year, then.tm_hour, then.tm_min, then.tm_sec);

    getfqhost (list->ehost, fqstring);
    
    if (linktype==LREPLINK) {
      fprintf   (datei, "<a href=\"");
      pCGIall   (datei, NULL, 0, FILELIST, 0, sort, repwhat);
      fprintf   (datei, "/rep%03d=host;%s\"",
		 numrep, cgiencode (fqstring));
      fprintf   (datei, ">%s</a> ", fqstring);
    }
    else
      fprintf   (datei, "%s ", fqstring);

    getfqfile (list->efile, fqstring);

    if (linktype==LREPLINK) {
      fprintf   (datei, "<a href=\"");
      pCGIall   (datei, NULL, 0, HOSTLIST, 0, sort, repwhat);
      fprintf   (datei, "/rep%03d=file;%s\"",
		 numrep, cgiencode (fqstring));
      fprintf   (datei, ">%s</a> ", fqstring);
    }
    else if (linktype==LFILELINK) {
      fprintf   (datei, "<a href=\"%s://%s:%d%s\"",
		 servtype, servhost, servport, fqstring);
      fprintf   (datei, ">%s</a> ", fqstring);
    }
    else
      fprintf   (datei, "%s ", fqstring);

    fprintf (datei, "\n");
    list = list->NEXT;
  }
  fprintf (datei, "</%s>", format);
}

void HTMLphsum (FILE *datei, int numrep, host *list)
{
  static char fqstring[1024];
  host *iter=list, *max;
  int maxreq=0, count=0, crtype;

  if ((display&HOSTLIST)==HOSTLIST)
    fprintf (datei, "<a name=\"HostSummary\"><h3>Top Level Domain Index</h3></a>\n");
  else
    fprintf (datei, "<a name=\"HostSummary\"><h3>Top Level Domain Summary</h3></a>\n");
  
  if (iter==NULL) {
    fprintf (datei, "<b>Sorry, this one is empty.</b><br>\n");
    return;
  }
  while (iter) {
    iter = iter->NEXT;
    count++;
  }
  iter = list;

  if (count==0) {
    fprintf (datei, "<b>Sorry, this one is empty.</b><br>\n");
    return;
  }

  if ((max = (host *) getmaxreq (list, HOSTTYPE, 0)) != NULL)
    maxreq = max->req;

  fprintf (datei, "<%s>", format);
  fprintf (datei, " Requests  kBytes : Domain%s",
	   Space (baroffset-strlen ("Domain"), ' '));
  if (maxreq&&barlength) {
    if (maxreq/barlength > 5)
      fprintf (datei,  "Requests (# = %d)", maxreq/barlength);
    else
      fprintf (datei,  "Requests (# = %d.%02d)", maxreq/barlength,
	       (100*maxreq)/barlength-100*(maxreq/barlength));
  }
  fprintf (datei, "\n");
  fprintf (datei, "------------------:-------%s\n",
	   Space(barlength+baroffset-strlen("Domain"),'-'));
  crtype = (display & ~HOSTLIST)|((display&FILESUM)?DATELIST:FILELIST);

  while (iter) {
    fprintf   (datei, " %8d %7ld : ", iter->req, iter->size>>1);
    getfqhost (iter,  fqstring);

    if ((display&HOSTLIST)==HOSTSUM && linktype==LREPLINK) {
      fprintf   (datei, "<a href=\"");
      pCGIall   (datei, NULL, 0, crtype, maxdepth, sort, repwhat);
      fprintf   (datei, "/rep%03d=host;%s\"",
		 numrep, cgiencode (fqstring));
      fprintf   (datei, ">%s</a>", iter->name);
    }
    else if ((display&HOSTLIST)==HOSTLIST && linktype!=LNOLINK) {
      fprintf   (datei, "<a href=\"#%s\">%s</a>", fqstring, iter->name);
    }
    else
      fprintf   (datei, "%s", iter->name);

    fprintf   (datei, "%s",  Space(baroffset-strlen(iter->name), ' '));
    if (barlength)
      fprintf (datei, "%s\n",Space(maxreq?(barlength*iter->req/maxreq):0,'#'));
    
    iter = iter->NEXT;
  }
  fprintf (datei, "</%s>\n", format);
}

void HTMLpfsum (FILE *datei, int numrep, file *list, int level)
{
  static char fqstring[1024];
  static int maxreq=0, crtype;
  file *iter=list, *max;
  char *dwhat;

  if (level == 1) {
    if ((display&FILELIST)==FILELIST) {
      fprintf (datei, "<a name=\"FileSummary\"><h3>Top Level Directory Index</h3></a>\n");
      dwhat = "Directory";
    }
    else {
      fprintf (datei, "<a name=\"FileSummary\"><h3>Top Level File Summary</h3></a>\n");
      dwhat = "File";
    }

    if (list == NULL || list->child == NULL) {
      fprintf (datei, "<b>Sorry, this one is empty.</b><br>\n");
      return;
    }

    crtype = (display & ~FILELIST)|((display&HOSTSUM)?DATELIST:HOSTLIST);

    if ((max = (file *) getmaxreq (list, FILETYPE, 1)) != NULL)
      maxreq = max->req;

    fprintf (datei, "<%s>", format);
    fprintf (datei, " Requests  kBytes : %s%s", dwhat,
	     Space (baroffset-strlen(dwhat), ' '));
    if (maxreq&&barlength) {
      if (maxreq/barlength > 5)
	fprintf (datei,  "Requests (# = %d)", maxreq/barlength);
      else
	fprintf (datei,  "Requests (# = %d.%02d)", maxreq/barlength,
		 (100*maxreq)/barlength-100*(maxreq/barlength));
    }
    fprintf (datei, "\n");
    fprintf (datei, "------------------:----------%s\n",
	     Space(barlength+baroffset-strlen(dwhat),'-'));
  }

  if (list==NULL)
    return;

  while (iter) {
    if (level > 0) {
      HTMLpfsum (datei, numrep, iter->child, level-1);
      iter = iter->NEXT;
      continue;
    }
    if (iter->isleaf && (display&FILELIST)!=FILESUM) {
      iter = iter->NEXT;
      continue;
    }
    fprintf   (datei, " %8d %7ld : ", iter->req, iter->size>>1);
    getfqfile (iter,  fqstring);

    if ((display&FILELIST)==FILESUM && linktype==LREPLINK) {
      fprintf   (datei, "<a href=\"");
      pCGIall   (datei, NULL, 0, crtype, maxdepth, sort, repwhat);
      fprintf   (datei, "/rep%03d=file;%s\"",
		 numrep, cgiencode (fqstring));
      fprintf   (datei, ">%s%s</a>", fqstring, (iter->isleaf)?"":"/");
    }
    else if ((display&FILELIST)==FILESUM && linktype==LFILELINK) {
      fprintf   (datei, "<a href=\"%s://%s:%d%s%s\"",
		 servtype, servhost, servport, fqstring,
		 (iter->isleaf)?"":"/");
      fprintf   (datei, ">%s%s</a>", fqstring, (iter->isleaf)?"":"/");
    }
    else if ((display&FILELIST)==FILELIST && linktype!=LNOLINK) {
      fprintf (datei, "<a href=\"#%s\">%s%s</a>",
	       fqstring, fqstring, (iter->isleaf)?"":"/");
    }
    else
      fprintf (datei, "%s%s", fqstring, (iter->isleaf)?"":"/");

    fprintf   (datei, "%s",  Space(baroffset-strlen(fqstring)-
				   ((iter->isleaf)?0:1), ' '));
    if (barlength)
      fprintf (datei, "%s\n",Space(maxreq?(barlength*iter->req/maxreq):0,'#'));

    iter = iter->NEXT;
  }
  if (level==1)
    fprintf (datei, "</%s>\n", format);
}

void HTMLpdsum (FILE *datei, int numrep, dtree *list, int level)
{
  static char fqstring[1024];
  dtree *iter=list, *max;
  static int maxreq=0, crtype;

  if (level == 2) {
    if ((display&DATELIST)==DATELIST)
      fprintf (datei, "<a name=\"DateSummary\"><h3>Top Level Date Index</h3></a>\n");
    else
      fprintf (datei, "<a name=\"DateSummary\"><h3>Top Level Date Summary</h3></a>\n");

    if (list == NULL) {
      fprintf (stdout, "<b>Sorry, this one's empty.</b><br>\n");
      return;
    }

    if ((max = (dtree *) getmaxreq (list, DATETYPE, 2)) != NULL)
      maxreq = max->req;
    else
      maxreq = 0;

    fprintf (datei, "<%s>", format);
    fprintf (datei, " Requests  kBytes : Day%s",
	     Space(baroffset-strlen("Day"), ' '));
    if (maxreq&&barlength) {
      if (maxreq/barlength > 10)
	fprintf (datei,  "Requests (# = %d)", maxreq/barlength);
      else
	fprintf (datei,  "Requests (# = %d.%02d)", maxreq/barlength,
		 (100*maxreq)/barlength-100*(maxreq/barlength));
    }
    fprintf (datei, "\n");
    fprintf (datei, "------------------:----%s\n",
	     Space(barlength+baroffset-strlen("Day"),'-'));

    crtype = (display & ~DATELIST)|((display&FILESUM)?HOSTLIST:FILELIST);
  }

  if (list == NULL)
    return;

  while (iter) {
    if (level > 0) {
      HTMLpdsum (datei, numrep, iter->child, level-1);
      iter = iter->NEXT;
      continue;
    }
    fprintf   (datei, " %8d %7ld : ", iter->req, iter->size>>1);
    getfqdate(iter,  fqstring);

    if ((display&DATELIST)==DATESUM && linktype==LREPLINK) {
      fprintf   (datei, "<a href=\"");
      pCGIall   (datei, NULL, 0, crtype, maxdepth, sort, repwhat);
      fprintf   (datei, "/rep%03d=host;%s\"",
		 numrep, cgiencode (fqstring));
      fprintf   (datei, ">%s</a>", iter->name);
    }
    else if ((display&DATELIST)==DATELIST && linktype!=LNOLINK) {
      fprintf   (datei, "<a href=\"#%s\">%s</a>", fqstring, iter->name);
    }
    else
      fprintf   (datei, "%s", iter->name);

    fprintf  (datei, "%s",  Space(baroffset-strlen(iter->name), ' '));
    if (barlength&&maxreq)
      fprintf(datei, "%s\n",Space(maxreq?(barlength*iter->req/maxreq):0,'#'));
    
    iter = iter->NEXT;
  }

  if (level==2)
    fprintf (datei, "</%s>\n", format);
}

/*
 * Main function of CGI part
 */

void CGImain (report *result, FILE *outfile)
{
  int numrep, ttype, count, inexc[4];
  char string[256], *p1, **ptr;
  char dat1[32], dat2[32];
  dtree *d1, *d2;

  htmlheader (outfile);

/*
 * debug test, dump form values
 */
#ifdef DEBUG
  fprintf (outfile, "<pre>\n");
  for (numrep=0;GetInputNameByNum(numrep);numrep++)
    fprintf (outfile, "'%s' = '%s'\n ",GetInputNameByNum(numrep),GetInputValueByNum(numrep));
  fprintf (outfile, "</pre>\n");
#endif

  if (GetInputValue ("SERVER_NAME")) {
    fprintf (outfile, "<!-- -->\n");
    fprintf (outfile, "<!-- Use the following URL to link to this page -->\n");
    fprintf (outfile, "<!-- \"");

    if ((p1 = GetInputValue ("SERVER_NAME")) != NULL) {
      fprintf (outfile, "http://%s", p1);
      if ((p1 = GetInputValue ("SERVER_PORT")) != NULL) {
	count = atoi (p1);
	if (count != 80)
	  fprintf (outfile, ":%d", count);
      }
    }
    pCGIall (outfile, NULL, 0, display, maxdepth, sort, repwhat);

    switch (linktype) {
    case LNOLINK:
      fprintf (outfile, "/Link=NoLink");
      break;
    case LFILELINK: 
      fprintf (outfile, "/Link=FileLink");
      if (strcmp (servhost, GetInputValue ("SERVER_NAME")))
	fprintf (outfile, "/Server=%s", servhost);
      if (strcmp (servtype, "http"))
	fprintf (outfile, "/ServerType=%s", servtype);
      if (servport != 80)
	fprintf (outfile, "/ServerPort=%d", servport);
      break;
    }
    fprintf (outfile, "\" -->\n");
    fprintf (outfile, "<!-- -->\n");
  }

  fprintf (outfile, "\n\
<hr>\n\
<h2>Search parameters</h2>\n\
<hr>\n");

  if ((inhost && *inhost) || (exhost && *exhost) ||
      (infile && *infile) || (exfile && *exfile)) {
    printf ("<b>Query limitations</b>");
    if (linktype==LREPLINK)
      fprintf (outfile, " (click to remove) (<a href=\"#morelimits\">add</a>)");
    fprintf (outfile, "\n<%s>", format);
  }
  else if (linktype==LREPLINK)
    fprintf (outfile, "Add some <a href=\"#morelimits\">query limitations</a><p>\n");

  inexc[0] = inexc[1] = inexc[2] = inexc[3] = 0;

  if ((ptr=inhost) && *inhost) {
    fprintf (outfile, "Limited to hosts:  ");
    while (*ptr) {
      if (linktype==LREPLINK) {
	fprintf (outfile, "<a href=\"");
	pCGIall (outfile, *ptr, 0, display, maxdepth, sort, repwhat);
	fprintf (outfile, "\">%s</a>", *ptr++);
      }
      else
	fprintf (outfile, "%s", *ptr++);
      if (*ptr) fprintf (outfile, "\n                   ");
      inexc[0] = inexc[0] + 1;
    }
    fprintf (outfile, "\n");
  }
  if ((ptr=exhost) && *exhost) {
    fprintf (outfile, "Hosts excluded:    ");
    while (*ptr) {
      if (linktype==LREPLINK) {
	fprintf (outfile, "<a href=\"");
	pCGIall (outfile, *ptr, 0, display, maxdepth, sort, repwhat);
	fprintf (outfile, "\">%s</a>", *ptr++);
      }
      else
	fprintf (outfile, "%s", *ptr++);
      if (*ptr) fprintf (outfile, "\n                   ");
      inexc[1] = inexc[1] + 1;
    }
    fprintf (outfile, "\n");
  }
  if ((ptr=infile) && *infile) {
    fprintf (outfile, "Limited to files:  ");
    while (*ptr) {
      if (linktype==LREPLINK) {
	fprintf (outfile, "<a href=\"");
	pCGIall (outfile, *ptr, 0, display, maxdepth, sort, repwhat);
	fprintf (outfile, "\">%s</a>", cgidecode (*ptr++));
      }
      else
	fprintf (outfile, "%s", *ptr++);
      if (*ptr) fprintf (outfile, "\n                   ");
      inexc[2] = inexc[2] + 1;
    }
    fprintf (outfile, "\n");
  }
  if ((ptr=exfile) && *exfile) {
    fprintf (outfile, "Files excluded:    ");
    while (*ptr) {
      if (linktype==LREPLINK) {
	fprintf (outfile, "<a href=\"");
	pCGIall (outfile, *ptr, 0, display, maxdepth, sort, repwhat);
	fprintf (outfile, "\">%s</a>", cgidecode (*ptr++));
      }
      else
	fprintf (outfile, "%s", *ptr++);
      if (*ptr) fprintf (outfile, "\n                   ");
      inexc[3] = inexc[3] + 1;
    }
    fprintf (outfile, "\n");
  }
  if ((inhost && *inhost) || (exhost && *exhost) ||
      (infile && *infile) || (exfile && *exfile))
    fprintf (outfile, "</%s>\n", format);

  numrep = 0;
  while (1) {
    if ((ttype = GetCGIReport (++numrep, string)) == NOTYPE)
      break;
    if (numrep==1) {
      fprintf (outfile, "<b>The following searches were applied</b>\n");
      if (linktype==LREPLINK)
	fprintf (outfile, "(click for <i>Apply to Global</i>)\n");
      fprintf (outfile, "<%s>\n", format);
    }
    
    switch (ttype) {
    case HOSTTYPE:
      if (linktype==LREPLINK) {
	fprintf  (outfile, "%3d  Host search:  <a href=\"", numrep);
	pCGIself (outfile, NULL);
	pCGIdisp (outfile, (display&~HOSTLIST)|((display&FILESUM)?DATELIST:FILELIST));
	pCGIdepth(outfile, maxdepth);
	pCGIsort (outfile, sort);
	pCGIrep  (outfile, repwhat);
	fprintf  (outfile, "/rep001=host;%s\">", cgiencode (string));
	fprintf  (outfile, "%s</a>",             cgidecode (string));
      }
      else
	fprintf  (outfile, "%3d  Host search:  %s\n",numrep,cgidecode(string));
      break;
    case FILETYPE:
      if (linktype==LREPLINK) {
	fprintf  (outfile, "%3d  File search:  <a href=\"", numrep);
	pCGIself (outfile, NULL);
	pCGIdisp (outfile, (display & ~FILELIST)|((display&HOSTSUM)?DATELIST:HOSTLIST));
	pCGIdepth(outfile, maxdepth);
	pCGIsort (outfile, sort);
	fprintf  (outfile, "/rep001=file;%s\">", cgiencode (string));
	fprintf  (outfile, "%s%s</a>", (*string)?"":"/", cgidecode (string));
      }
      else
	fprintf  (outfile, "%3d  File search:  %s%s\n",
		  numrep, (*string)?"":"/", cgidecode(string));
      break;
    case DATETYPE:
      if (linktype==LREPLINK) {
	fprintf  (outfile, "%3d  Date search:  <a href=\"", numrep);
	pCGIself (outfile, NULL);
	pCGIdisp (outfile, (display & ~DATELIST)|((display&FILESUM)?HOSTLIST:FILELIST));
	pCGIdepth(outfile, maxdepth);
	pCGIsort (outfile, sort);
	fprintf  (outfile, "/rep001=date;%s\">", cgiencode (string));
	fprintf  (outfile, "%s</a>",             cgidecode (string));
      }
      else
	fprintf  (outfile, "%3d  Date search:  %s\n",numrep,cgidecode(string));
      break;
    }
    if (linktype==LREPLINK) {
      fprintf  (outfile, " (<a href=\"");
      pCGIall  (outfile, NULL, numrep, display, maxdepth, sort, repwhat);
      fprintf  (outfile, "\">remove</a>)");
    }
    fprintf (outfile, "\n");
  }
  if (numrep>1)
    fprintf (outfile, "</%s>\n", format);

  fprintf (outfile, "This report covers ");
  switch (repwhat) {
  case RTODAY:      fprintf (outfile, "today");              break;
  case RYESTERDAY:  fprintf (outfile, "yesterday");          break;
  case RTHISWEEK:   fprintf (outfile, "this week");          break;
  case RLASTWEEK:   fprintf (outfile, "last week");          break;
  case RTHISMONTH:  fprintf (outfile, "this month");         break;
  case RLASTMONTH:  fprintf (outfile, "last month");         break;
  case REVERYTHING: fprintf (outfile, "all available dates");break;
  case RLAST24HOURS:fprintf (outfile, "the last 24 hours");  break;
  case RLAST7DAYS:  fprintf (outfile, "the last 7 days");    break;
  case RDATES:      fprintf (outfile, "these dates:");       break;
  default:          fprintf (outfile, "??? (this shouldn't happen)"); break;
  }
  if ((d1=result->mindate) && (d2=result->maxdate)) {
    while (d1->depth > ((repwhat==RLAST24HOURS) ? 4 : 2)) d1 = d1->parent;
    while (d2->depth > ((repwhat==RLAST24HOURS) ? 4 : 2)) d2 = d2->parent;
    if (d1 == d2)
      fprintf (outfile, " (%s)",   getfqdate (d1, string));
    else {
      fprintf (outfile, " (%s to", getfqdate (d1, string));
      fprintf (outfile, " %s)",    getfqdate (d2, string));
    }
  }

  fprintf (outfile, ".<br>\n");

  fprintf (outfile, "The following is printed:<br>\n");
  fprintf (outfile, "<blockquote>\n");

  switch (display&HOSTLIST) {
  case 0:        fprintf (outfile, "No Host Listing");   break;
  case HOSTSUM:  fprintf (outfile, "Host Summary");      break;
  case HOSTLIST: fprintf (outfile, "Full Host Listing"); break;
  }
  if (linktype==LREPLINK) {
    if ((display&HOSTLIST)==HOSTLIST) {
      fprintf (outfile, " (Detail: ");
      for (count=0; count<6; count++) {
	if (count==HOSTDEPTH(maxdepth))
	  continue;
	fprintf (outfile, "<a href=\"");
	pCGIall (outfile, NULL, 0, display,
		 (maxdepth&~HOSTDETMASK)|(count<<HOSTDETOFF),
		 sort, repwhat);
	fprintf (outfile, "\">");
	if (count==0) fprintf (outfile, "All");
	else          fprintf (outfile, "%d", count);
	fprintf (outfile, "</a> ");
      }
      fprintf (outfile, "Levels)");
    }
    fprintf (outfile, " (");
    if ((display&HOSTLIST)!=0) {
      fprintf (outfile, "<a href=\"");
      pCGIall (outfile, NULL, 0, (display&~(HOSTLIST|ENTRYLIST)), maxdepth, sort, repwhat);
      fprintf (outfile, "\">No Listing</a> ");
    }
    if ((display&HOSTLIST)!=HOSTSUM) {
      fprintf (outfile, "<a href=\"");
      pCGIall (outfile, NULL, 0, (display&~(HOSTLIST|ENTRYLIST))|HOSTSUM, maxdepth, sort, repwhat);
      fprintf (outfile, "\">Summary only</a> ");
    }
    if ((display&HOSTLIST)!=HOSTLIST) {
      fprintf (outfile, "<a href=\"");
      pCGIall (outfile, NULL, 0,
	       (display&~ENTRYLIST)|HOSTLIST,
	       maxdepth&~HOSTDETMASK, sort, repwhat);
      fprintf (outfile, "\">Full Listing</a> ");
    }
    fprintf (outfile, ")<br>\n");
  }
  else
    fprintf (outfile, ", ");
    
  switch (display&FILELIST) {
  case 0:        fprintf (outfile, "No File Listing");        break;
  case FILESUM:  fprintf (outfile, "Top Level File Summary"); break;
  case FILELIST: fprintf (outfile, "Full File Listing");      break;
  }
  if (linktype==LREPLINK) {
    if ((display&FILELIST)==FILELIST) {
      fprintf (outfile, " (Detail: ");
      for (count=0; count<6; count++) {
	if (count==FILEDEPTH(maxdepth))
	  continue;
	fprintf (outfile, "<a href=\"");
	pCGIall (outfile, NULL, 0, display,
		 (maxdepth&~FILEDETMASK)|(count<<FILEDETOFF),
		 sort, repwhat);
	fprintf (outfile, "\">");
	if (count==0) fprintf (outfile, "All");
	else          fprintf (outfile, "%d", count);
	fprintf (outfile, "</a> ");
      }
      fprintf (outfile, "Levels)");
    }
    fprintf (outfile, " (");
    if ((display&FILELIST)!=0) {
      fprintf (outfile, "<a href=\"");
      pCGIall (outfile, NULL, 0, (display&~(FILELIST|ENTRYLIST)), maxdepth, sort, repwhat);
      fprintf (outfile, "\">No Listing</a> ");
    }
    if ((display&FILELIST)!=FILESUM) {
      fprintf (outfile, "<a href=\"");
      pCGIall (outfile, NULL, 0, (display&~(FILELIST|ENTRYLIST))|FILESUM, maxdepth, sort, repwhat);
      fprintf (outfile, "\">Summary only</a> ");
    }
    if ((display&FILELIST)!=FILELIST) {
      fprintf (outfile, "<a href=\"");
      pCGIall (outfile, NULL, 0,
	       (display&~ENTRYLIST)|FILELIST,
	       maxdepth&~FILEDETMASK, sort, repwhat);
      fprintf (outfile, "\">Full Listing</a> ");
    }
    fprintf (outfile, ")<br>\n");
  }
  else
    fprintf (outfile, ", ");
    
  switch (display&DATELIST) {
  case 0:        fprintf (outfile, "No Date Listing");   break;
  case DATESUM:  fprintf (outfile, "Daily Summary");     break;
  case DATELIST: fprintf (outfile, "Full Date Listing"); break;
  }
  if (linktype == LREPLINK) {
    if ((display&DATELIST)==DATELIST) {
      fprintf (outfile, " (Detail: ");
      for (count=0; count<5; count++) {
	if (count==DATEDEPTH(maxdepth))
	  continue;
	fprintf (outfile, "<a href=\"");
	pCGIall (outfile, NULL, 0, display,
		 (maxdepth&~DATEDETMASK)|(count<<DATEDETOFF),
		 sort, repwhat);
	fprintf (outfile, "\">");
	fprintf (outfile, "%s",
		 ((count==0)?"All":
		  ((count==1)?"Year":
		   ((count==2)?"Month":
		    ((count==3)?"Day":
		     ((count==4)?"Hour":"This is a bug"))))));
	fprintf (outfile, "</a> ");
      }
      fprintf (outfile, ")");
    }
    fprintf (outfile, " (");
    if ((display&DATELIST)!=0) {
      fprintf (outfile, "<a href=\"");
      pCGIall (outfile, NULL, 0, (display&~(DATELIST|ENTRYLIST)), maxdepth, sort, repwhat);
      fprintf (outfile, "\">No Listing</a> ");
    }
    if ((display&DATELIST)!=DATESUM) {
      fprintf (outfile, "<a href=\"");
      pCGIall (outfile, NULL, 0, (display&~(DATELIST|ENTRYLIST))|DATESUM, maxdepth, sort, repwhat);
      fprintf (outfile, "\">Daily Summary</a> ");
    }
    if ((display&DATELIST)!=DATELIST) {
      fprintf (outfile, "<a href=\"");
      pCGIall (outfile, NULL, 0,
	       (display&~ENTRYLIST)|DATELIST,
	       maxdepth&~DATEDETMASK, sort, repwhat);
      fprintf (outfile, "\">Full Listing</a> ");
    }
    fprintf (outfile, ")<br>\n");
  }
  else
    fprintf (outfile, "<br>\n");
    
  if (linktype==LREPLINK) {
    if ((display&ENTRYLIST)!=ENTRYLIST) {
      fprintf (outfile, "<a href=\"");
      pCGIall (outfile, NULL, 0, ENTRYLIST, maxdepth, sort, repwhat);
      fprintf (outfile, "\">Entry Listing</a> (exclusive)");
    }
  }
  fprintf (outfile, "</blockquote>");
  
  fprintf (outfile, "This report is sorted ");
  switch (sort) {
  case SORTALPHA: fprintf (outfile, "alphabetically"); break;
  case SORTREQ:   fprintf (outfile, "by requests");    break;
  case SORTSIZE:  fprintf (outfile, "by size");        break;
  default:        fprintf (outfile, "??? (this shouldn't happen)"); break;
  }
  if (linktype==LREPLINK) {
    fprintf (outfile, " (or sort ");
    if (sort != SORTALPHA) {
      fprintf  (outfile, "<a href=\"");
      pCGIall  (outfile, NULL, 0, display, maxdepth, SORTALPHA, repwhat);
      fprintf  (outfile, "\">alphabetically</a> \n");
    }
    if (sort != SORTREQ) {
      fprintf  (outfile, "<a href=\"");
      pCGIall  (outfile, NULL, 0, display, maxdepth, SORTREQ,   repwhat);
      fprintf  (outfile, "\">by requests</a> \n");
    }
    if (sort != SORTSIZE) {
      fprintf  (outfile, "<a href=\"");
      pCGIall  (outfile, NULL, 0, display, maxdepth, SORTSIZE,  repwhat);
      fprintf  (outfile, "\">by size</a> \n");
    }
    fprintf (outfile, ")");
  }
  fprintf (outfile, ".<br>\n");

#ifdef GMT
  fprintf (outfile, "Times are reported in GMT.<br>\n");
#else
  fprintf (outfile, "Times are reported in local time.<br>\n");
#endif

  if (display&HOSTLIST || countunique)
    fprintf (outfile, "%d unique %s were found.<br>\n",
	     countleafs (result->hlist, HOSTTYPE, -1),
	     ((HOSTDEPTH(maxdepth)==0 && (display&HOSTLIST)==HOSTLIST) ||
	      countunique) ? "hosts" : "domains");
  
  fprintf (outfile, "\
<hr>\n\
<h1><a name=\"Results\">Search results</a></h1>\n\
<dl>\n\
<dt> <b>Quick Index</b>\n");
  if (((display&HOSTSUM)  == HOSTSUM) &&
      ((display&HOSTLIST) != HOSTLIST ||
       HOSTDEPTH(maxdepth) > 1 ||
       HOSTDEPTH(maxdepth)== 0))
    fprintf (outfile, "<dd> <a href=\"#HostSummary\">Host Summary</a>\n");
  if ((display&HOSTLIST) == HOSTLIST)
    fprintf (outfile, "<dd> <a href=\"#HostListing\">Host Listing</a>\n");

  if (((display&FILESUM)  == FILESUM) &&
      ((display&FILELIST) != FILELIST ||
       FILEDEPTH(maxdepth) > 2 ||
       FILEDEPTH(maxdepth)== 0))
    fprintf (outfile, "<dd> <a href=\"#FileSummary\">File Summary</a>\n");
  if ((display&FILELIST) == FILELIST)
    fprintf (outfile, "<dd> <a href=\"#FileListing\">File Listing</a>\n");

  if (((display&DATESUM)  == DATESUM) &&
      ((display&DATELIST) != DATELIST || 
       DATEDEPTH(maxdepth) > 2 ||
       DATEDEPTH(maxdepth)== 0))
    fprintf (outfile, "<dd> <a href=\"#DateSummary\">Date Summary</a>\n");
  if ((display&DATELIST) == DATELIST)
    fprintf (outfile, "<dd> <a href=\"#DateListing\">Date Listing</a>\n");

  if (display&ENTRYLIST)
    fprintf (outfile, "<dd> <a href=\"#EntryListing\">Entry Listing</a>\n");
  fprintf (outfile, "</dl>\n");

  if (display&HOSTLIST) {
    fprintf (outfile, "\n<hr>\n<h2>Output by Host</h2>\n<hr>\n");
    if (((display&HOSTSUM)  == HOSTSUM) &&
	((display&HOSTLIST) != HOSTLIST ||
	 HOSTDEPTH(maxdepth) > 1 ||
	 HOSTDEPTH(maxdepth)== 0))
      HTMLphsum  (outfile, numrep, result->hlist);
    if ((display&HOSTLIST) == HOSTLIST)
      HTMLphosts (outfile, numrep, result->hlist, 0);
  }

  if (display&FILELIST) {
    fprintf (outfile, "\n<hr>\n<h2>Output by File</h2>\n<hr>\n");
    if (((display&FILESUM)  == FILESUM) &&
	((display&FILELIST) != FILELIST ||
	 FILEDEPTH(maxdepth) > 2 ||
	 FILEDEPTH(maxdepth)== 0))
      HTMLpfsum  (outfile, numrep, result->flist, 1);
    if ((display&FILELIST) == FILELIST)
      HTMLpfiles (outfile, numrep, result->flist, 0);
  }

  if (display&DATELIST) {
    fprintf (outfile, "\n<hr>\n<h2>Output by Date</h2>\n<hr>\n");
    if (((display&DATESUM)  == DATESUM) &&
	((display&DATELIST) != DATELIST || 
	 DATEDEPTH(maxdepth) > 2 ||
	 DATEDEPTH(maxdepth)== 0))
      HTMLpdsum  (outfile, numrep, result->dlist, 2);
    if ((display&DATELIST) == DATELIST)
      HTMLpdates (outfile, numrep, result->dlist, 0);
  }

  if (display&ENTRYLIST) {
    fprintf (outfile, "\n<hr>\n<h2>Output by Entry</h2>\n<hr>\n");
    if ((display&ENTRYLIST)== ENTRYLIST)
      HTMLpents  (outfile, numrep, result->elist, 0);
  }
  fprintf (outfile, "<p>\n");

  if (linktype==LREPLINK) {
    if (repwhat == RDATES) {
      sprintf (dat1, "%s %02d %02d",
	       mnames[fromdate.tm_mon+1], fromdate.tm_mday, fromdate.tm_year);
      sprintf (dat2, "%s %02d %02d",
	       mnames[todate.tm_mon+1],   todate.tm_mday, todate.tm_year);
    }
    fprintf (outfile, "\
\n\
<hr>\n\
<b><a name=\"morelimits\">Add more query limitations</a></b><br>\n\
<FORM METHOD=\"POST\" ACTION=\"");
    
    pCGIall (outfile, NULL, 0, display, maxdepth, sort, -1);

    fprintf (outfile,"\">\n\
<KBD>Include hosts: </KBD><input name=\"inhost%03d\" size=\"40\"><br>\n\
<KBD>Exclude hosts: </KBD><input name=\"exhost%03d\" size=\"40\"><br>\n\
<KBD>Include files: </KBD><input name=\"infile%03d\" size=\"40\"><br>\n\
<KBD>Exclude files: </KBD><input name=\"exfile%03d\" size=\"40\"><br>\n\
Report same on\n\
<select name=\"Report\">\n\
<option %s value=\"%s\">Today\n\
<option %s value=\"%s\">Yesterday\n\
<option %s value=\"%s\">This Week\n\
<option %s value=\"%s\">Last Week\n\
<option %s value=\"%s\">This Month\n\
<option %s value=\"%s\">Last Month\n\
<option %s value=\"%s\">Last 24 Hours\n\
<option %s value=\"%s\">Last 7 Days\n\
<option %s value=\"%s\">Everything\n\
<option %s value=\"%s\">These Dates:\n\
</select>\n",
	     inexc[0]+1, inexc[1]+1, inexc[2]+1, inexc[3]+1,
	     (repwhat == RTODAY)      ? "selected" : "", repnames[RTODAY],
	     (repwhat == RYESTERDAY)  ? "selected" : "", repnames[RYESTERDAY],
	     (repwhat == RTHISWEEK)   ? "selected" : "", repnames[RTHISWEEK],
	     (repwhat == RLASTWEEK)   ? "selected" : "", repnames[RLASTWEEK],
	     (repwhat == RTHISMONTH)  ? "selected" : "", repnames[RTHISMONTH],
	     (repwhat == RLASTMONTH)  ? "selected" : "", repnames[RLASTMONTH],
	     (repwhat == RLAST24HOURS)? "selected" : "",repnames[RLAST24HOURS],
	     (repwhat == RLAST7DAYS)  ? "selected" : "", repnames[RLAST7DAYS],
	     (repwhat == REVERYTHING) ? "selected" : "", repnames[REVERYTHING],
	     (repwhat == RDATES)      ? "selected" : "", repnames[RDATES]);

    fprintf (outfile, "\
From <input value=\"%s\" name=\"FromDate\" size=\"10\">\n\
To <input value=\"%s\" name=\"ToDate\" size=\"10\">\n\
<br>\n\
<INPUT type=submit value=\"Apply\"> these additions\n\
</FORM>\n\
\n",
	     (repwhat == RDATES)      ? dat1 : "",
	     (repwhat == RDATES)      ? dat2 : "");

    fprintf    (outfile, "\n");
    fprintf    (outfile, "Return to <a href=\"");
    pCGIself   (outfile, NULL);
    pCGIdisp   (outfile, display);
    pCGIdepth  (outfile, maxdepth);
    pCGIsort   (outfile, sort);
    pCGIrep    (outfile, repwhat);
    fprintf    (outfile, "\">Global</a><br>\n");
    fprintf    (outfile, "Return to <a href=\"");
    pCGIself   (outfile, NULL);
    pCGIdisp   (outfile, display);
    pCGIdepth  (outfile, maxdepth);
    pCGIsort   (outfile, sort);
    pCGIrep    (outfile, repwhat);
    fprintf    (outfile, "/form\">Query Form</a>\n");
  } /* if nolink */

  htmlfooter (outfile);
}
