/*
 * This file is part of W3Olista, the statistics gathering program for
 * the WWW.                  This sourcecode is (c) Frank Pilhofer '95
 *
 * It is  distributed without warranty. The complete license terms for
 * this software  are included in  the file 'License.html'  in the doc
 * subdirectory. If  your copy came  without this file, you should get
 * it on the Web at    http://www.uni-frankfurt.de/~fp/Tools/W3Olista/
 * or contact the author,               fp@informatik.uni-frankfurt.de
 */

#ifdef SHMONITOR
#include <stdio.h>
#include <memory.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>
#include "monitor.h"
#include "struct.h"

static shmonitor *shmem=NULL;
static int        shmid=0;

void handler (int sig)
{
  if (shmem)
    shmdt ((void *)shmem);
  exit (0);
}

int main (void)
{
  int flag=0, wrirem=0;
  struct shmid_ds buf;
  time_t now;

  printf ("Beginning monitoring ...\n");
  signal (SIGINT, handler);

  while (1) {
    if ((shmid = shmget (SHMONITOR, sizeof (shmonitor), 0777)) == -1) {
      if (flag)
	printf ("Shared memory was destroyed.\n");
      sleep (1);
      flag = 0;
      continue;
    }
    if (flag==0) {
      printf ("Shared memory found.\n");
      wrirem = 0;
      flag   = 1;
    }

    if ((shmem = (shmonitor *) shmat (shmid, NULL, 0)) == NULL) {
      sleep  (1);
      continue;
    }

    if (shmem->cookie != 6274) {
      printf ("Oops. Wrong cookie. Waiting 2 secs ...\n");
      sleep  (2);
      if (shmem->cookie != 6274) {
	printf ("Still wrong :-((\n");
	shmdt  ((void *) shmem);
	fflush (stdout);
	shmem = NULL;
	continue;
      }
    }

    if (shmctl (shmid, IPC_STAT, &buf) != -1) {
      if (buf.shm_nattch == 1) {
	printf ("Oops. Correctly initialized shmem found, but not attached.\n");
	shmdt  ((void *) shmem);
	shmctl (shmid, IPC_RMID, NULL);
	fflush (stdout);
	shmem = NULL;
	continue;
      }
    }
    
    if (wrirem==0) {
      now = time(NULL);
      printf ("Called from %s at %s", shmem->remote, ctime(&now));
      wrirem = 1;
    }

    if (shmem->haschanged == 0) {
      sleep (1);
      shmdt ((void *) shmem);
      shmem = NULL;
      continue;
    }
    switch (shmem->flag) {
    case MF_INIT:
      printf ("Olista is initializing\n");
      break;
    case MF_LOADING:
      printf ("loading %7d/%-7d  line %6d : %s heap=%7d\n",
	      shmem->totline, shmem->hopline,
	      shmem->momline, shmem->momfile,
	      shmem->heapused);
      break;
    case MF_REPORT:
      printf ("generating report on ");
      switch (shmem->reptype) {
      case HOSTTYPE:
	printf ("host");
	break;
      case FILETYPE:
	printf ("file");
	break;
      case DATETYPE:
	printf ("date");
	break;
      default:
	printf ("????");
	break;
      }
      printf (" %s\n", shmem->momfile);
      break;
    case MF_WRITE:
      printf ("Writing report.\n");
      break;
    }
    shmem->haschanged = 0;
    sleep (1);
    shmdt ((void *) shmem);
    shmem = NULL;
    fflush (stdout);
  }
  return 0;
}

#else /* if SHMONITOR */

#include <stdio.h>

int main (void)
{
  printf ("W3Olista was compiled without monitor support.\n");
  return 0;
}

#endif /* SHMONITOR */
